﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media.Media3D;
using Infragistics.Controls.Charts;

namespace XamDataChart
{

    #region Chart tooltip behavior
    public class ChartTooltipBehaviors : DependencyObject
    {
        public static readonly DependencyProperty CursorTooltipProperty =
            DependencyProperty.RegisterAttached("CursorTooltip",
            typeof(CursorTooltipBehavior), typeof(ChartTooltipBehaviors),
            new PropertyMetadata(null,
                (o, e) => CursorTooltipChanged(
                    o as XamDataChart,
                    e.OldValue as CursorTooltipBehavior,
                    e.NewValue as CursorTooltipBehavior)));

        public static CursorTooltipBehavior GetCursorTooltip(
            DependencyObject target)
        {
            return target.GetValue(CursorTooltipProperty)
                as CursorTooltipBehavior;
        }

        public static void SetCursorTooltip(
            DependencyObject target, CursorTooltipBehavior behavior)
        {
            target.SetValue(CursorTooltipProperty, behavior);
        }

        private static void CursorTooltipChanged(
            XamDataChart chart,
            CursorTooltipBehavior oldValue,
            CursorTooltipBehavior newValue)
        {
            if (chart == null)
            {
                return;
            }
            if (oldValue != null)
            {
                oldValue.OnDetach(chart);
            }
            if (newValue != null)
            {
                newValue.OnAttach(chart);
            }
        }
    }

    public class SeriesItemInfo
        : INotifyPropertyChanged
    {
        private Series _series;
        public Series Series
        {
            get { return _series; }
            set
            {
                _series = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(
                        this,
                        new PropertyChangedEventArgs("Series"));
                }
            }
        }

        private object _item;
        public object Item
        {
            get { return _item; }
            set
            {
                _item = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(
                        this,
                        new PropertyChangedEventArgs("Item"));
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class SeriesItemInfoCollection
        : ObservableCollection<SeriesItemInfo>
    {
        public void UpdateSeriesItem(
            Series series,
            object item)
        {
            bool found = false;
            var indexes = from curr in this
                          where curr.Series == series
                          select this.IndexOf(curr);
            foreach (int index in indexes)
            {
                found = true;
                this[index].Item = item;
            }

            if (!found)
            {
                this.Add(
                    new SeriesItemInfo()
                    {
                        Series = series,
                        Item = item
                    });
            }
        }
    }

    public class CursorTooltipBehavior
    {
        private bool _isOverChart = false;
        private Popup _popup = new Popup();

        private ContentControl _content =
            new ContentControl();
        private Panel _container;
        private XamDataChart _owner = null;

        private DataTemplate _tooltipTemplate;

        private SeriesItemInfoCollection _items
            = new SeriesItemInfoCollection();

        public DataTemplate TooltipTemplate
        {
            get { return _tooltipTemplate; }
            set
            {
                _tooltipTemplate = value;
                _content.ContentTemplate = _tooltipTemplate;
            }
        }

        protected bool IsOverChart
        {
            get { return _isOverChart; }
            set
            {
                bool last = _isOverChart;
                _isOverChart = value;
                if (_isOverChart && !last)
                {
                    ShowPopup();
                }
                if (!_isOverChart && last)
                {
                    HidePopup();
                }
            }
        }

        private void HidePopup()
        {
            _popup.IsOpen = false;
        }

        private void ShowPopup()
        {
            _popup.Placement = PlacementMode.Absolute;
            _popup.IsOpen = true;
        }

        public void OnAttach(XamDataChart chart)
        {
            if (_owner != null)
            {
                OnDetach(_owner);
            }
            chart.MouseLeave += Chart_MouseLeave;
            chart.MouseMove += Chart_MouseMove;
            chart.SeriesCursorMouseMove +=
                Chart_SeriesCursorMouseMove;

            _popup.IsOpen = false;
            _popup.Child = _content;
            _content.ContentTemplate = TooltipTemplate;
            _content.Content = _items;
            if (chart.Parent != null && chart.Parent is Panel)
            {
                _container = chart.Parent as Panel;
                _container.Children.Add(_popup);
            }
        }

        public void OnDetach(XamDataChart chart)
        {
            if (_owner != chart)
            {
                return;
            }
            chart.MouseLeave -= Chart_MouseLeave;
            chart.MouseMove -= Chart_MouseMove;
            chart.SeriesCursorMouseMove -=
                Chart_SeriesCursorMouseMove;
            IsOverChart = false;
            _items.Clear();
            _owner = null;
        }

        void Chart_MouseMove(object sender, MouseEventArgs e)
        {
            //SetPopupOffsets(e.GetPosition(_owner));
            //IsOverChart = true;
            double x = e.GetPosition(sender as XamDataChart).X + (sender as XamDataChart).PointToScreen(new Point(0, 0)).X;
            double y = e.GetPosition(sender as XamDataChart).Y + (sender as XamDataChart).PointToScreen(new Point(0, 0)).Y;
            Point p = new Point(x, y);
            SetPopupOffsets(p);
            IsOverChart = true;
        }

        private void SetPopupOffsets(Point point)
        {
            _popup.VerticalOffset = point.Y;
            _popup.HorizontalOffset = point.X + 10;
        }

        void Chart_SeriesCursorMouseMove(
            object sender, ChartCursorEventArgs e)
        {
            if (e.Series != null && e.Item != null)
            {
                _items.UpdateSeriesItem(e.Series, e.Item);
            }
        }

        void Chart_MouseLeave(
            object sender, MouseEventArgs e)
        {
            IsOverChart = false;
        }
    }
    #endregion End chart tooltip behavior



    #region XamChart 3d View Rotations
    public class UperLeftDownRightAxisConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            double result1;
            double result2;
            if (Double.TryParse(values[0].ToString(), out result1) && Double.TryParse(values[1].ToString(), out result2))
            {
                return new Vector3D((double)values[0], (double)values[1], 0);
            }
            return Binding.DoNothing;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class UperRightDownLeftAxisConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            double result1;
            double result2;
            if (Double.TryParse(values[0].ToString(), out result1) && Double.TryParse(values[1].ToString(), out result2))
            {
                return new Vector3D((double)values[0], -(double)values[1], 0);
            }
            return Binding.DoNothing;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class RotateLeftCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[1] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle -= 2;
        }
    }

    public class RotateRightCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[1] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

    public class RotateUpCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[0] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle -= 2;
        }
    }

    public class RotateDownCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[0] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

    public class RotateUpLeftCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[2] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle -= 2;
        }
    }

    public class RotateDownRightCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[2] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

    public class RotateUpRightCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[3] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle -= 2;
        }
    }

    public class RotateDownLeftCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[3] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

    public class RotateCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            //((((parameter as Infragistics.Windows.Chart.XamChart).Transform3D as Transform3DGroup).Children[4] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }
    #endregion

    #region XamChart Axis Titles
    public static class AxisInfo
    {
        public static readonly DependencyProperty XAxisTitleProperty =
            DependencyProperty.RegisterAttached(
            "XAxisTitle",
            typeof(string),
            typeof(AxisInfo),
            new PropertyMetadata(""));

        public static string GetXAxisTitle(
            DependencyObject target)
        {
            return (string)target.GetValue(XAxisTitleProperty);
        }

        public static void SetXAxisTitle(
            DependencyObject target,
            string title)
        {
            target.SetValue(XAxisTitleProperty, title);
        }

        public static readonly DependencyProperty YAxisTitleProperty =
            DependencyProperty.RegisterAttached(
            "YAxisTitle",
            typeof(string),
            typeof(AxisInfo),
            new PropertyMetadata(""));

        public static string GetYAxisTitle(
            DependencyObject target)
        {
            return (string)target.GetValue(YAxisTitleProperty);
        }

        public static void SetYAxisTitle(
            DependencyObject target,
            string title)
        {
            target.SetValue(YAxisTitleProperty, title);
        }
    }
    #endregion



}
